@@ -12,7 +12,6 @@
 #include "nsPrintfCString.h"
 #include "nsString.h"
 #include "mozilla/Components.h"
-#include "mozilla/Preferences.h"
 #include "nsIRequest.h"
 #include "nsIStringBundle.h"
 #include "nsIURI.h"
@@ -26,8 +25,6 @@
 #include "nsIPgpMimeProxy.h"
 #include "nsComponentManagerUtils.h"
 
-using mozilla::Preferences;
-
 /**
  * Overall description
  * ===================
