@@ -63,7 +63,6 @@
 #include "mozilla/CheckedInt.h"
 #include "mozilla/NullPrincipal.h"
 #include "mozilla/Preferences.h"
-#include "mozilla/ResultExtensions.h"
 #include "mozilla/StaticPrefs_browser.h"
 #include "mozilla/StaticPrefs_middlemouse.h"
 #include "mozilla/StaticPrefs_full_screen_api.h"
@@ -91,15 +90,8 @@ class nsWindowWatcher;
 struct nsWatcherWindowEntry {
   nsWatcherWindowEntry(mozIDOMWindowProxy* aWindow,
                        nsIWebBrowserChrome* aChrome)
-      : mChrome(nullptr) {
-    mWindow = aWindow;
-    nsCOMPtr<nsISupportsWeakReference> supportsweak(do_QueryInterface(aChrome));
-    if (supportsweak) {
-      supportsweak->GetWeakReference(getter_AddRefs(mChromeWeak));
-    } else {
-      mChrome = aChrome;
-      mChromeWeak = nullptr;
-    }
+      : mWindow(do_GetWeakReference(aWindow)),
+        mChrome(do_GetWeakReference(aChrome)) {
     ReferenceSelf();
   }
   ~nsWatcherWindowEntry() = default;
@@ -108,9 +100,8 @@ struct nsWatcherWindowEntry {
   void Unlink();
   void ReferenceSelf();
 
-  mozIDOMWindowProxy* mWindow;
-  nsIWebBrowserChrome* mChrome;
-  nsWeakPtr mChromeWeak;
+  nsWeakPtr mWindow;
+  nsWeakPtr mChrome;
   // each struct is in a circular, doubly-linked list
   nsWatcherWindowEntry* mYounger;  // next younger in sequence
   nsWatcherWindowEntry* mOlder;
@@ -194,10 +185,15 @@ nsWatcherWindowEnumerator::GetNext(nsISupports** aResult) {
 
   *aResult = nullptr;
 
-  if (mCurrentPosition) {
-    CallQueryInterface(mCurrentPosition->mWindow, aResult);
+  while (mCurrentPosition) {
+    nsCOMPtr<mozIDOMWindowProxy> window =
+        do_QueryReferent(mCurrentPosition->mWindow);
+    if (window) {
+      CallQueryInterface(window, aResult);
+      mCurrentPosition = FindNext();
+      return NS_OK;
+    }
     mCurrentPosition = FindNext();
-    return NS_OK;
   }
   return NS_ERROR_FAILURE;
 }
@@ -554,8 +550,8 @@ nsWindowWatcher::OpenWindowWithRemoteTab(
   // don't need to propagate isPopupRequested out-parameter to the resulting
   // browsing context.
   bool unused = false;
-  uint32_t chromeFlags =
-      CalculateChromeFlagsForContent(aFeatures, aModifiers, &unused);
+  uint32_t chromeFlags = CalculateChromeFlagsForContent(aFeatures, aModifiers,
+                                                        aCalledFromJS, &unused);
 
   if (isPrivateBrowsingWindow) {
     chromeFlags |= nsIWebBrowserChrome::CHROME_PRIVATE_WINDOW;
@@ -805,8 +801,8 @@ nsresult nsWindowWatcher::OpenWindowInternal(
   } else {
     MOZ_DIAGNOSTIC_ASSERT(parentBC && parentBC->IsContent(),
                           "content caller must provide content parent");
-    chromeFlags =
-        CalculateChromeFlagsForContent(features, aModifiers, &isPopupRequested);
+    chromeFlags = CalculateChromeFlagsForContent(
+        features, aModifiers, aCalledFromJS, &isPopupRequested);
 
     if (aDialog) {
       MOZ_ASSERT(XRE_IsParentProcess());
@@ -887,43 +883,43 @@ nsresult nsWindowWatcher::OpenWindowInternal(
          : nsContentUtils::GetSystemPrincipal();
   MOZ_ASSERT(subjectPrincipal);
 
-  nsCOMPtr<nsIPrincipal> newWindowPrincipal;
+  // Information used when opening new content windows. This object will be
+  // passed through to the inner nsFrameLoader.
+  RefPtr<nsOpenWindowInfo> openWindowInfo = new nsOpenWindowInfo();
+
   if (!targetBC) {
     if (windowTypeIsChrome) {
       // If we are creating a chrome window, we must be called with a system
       // principal, and should inherit that for the new chrome window.
       MOZ_RELEASE_ASSERT(subjectPrincipal->IsSystemPrincipal(),
                          "Only system principals can create chrome windows");
-      newWindowPrincipal = subjectPrincipal;
+      openWindowInfo->mPrincipalToInheritForAboutBlank = subjectPrincipal;
     } else if (nsContentUtils::IsSystemOrExpandedPrincipal(subjectPrincipal)) {
       // Don't allow initial about:blank documents to inherit a system or
       // expanded principal, instead replace it with a null principal. We can't
       // inherit origin attributes from the system principal, so use the parent
       // BC if it's available.
       if (parentBC) {
-        newWindowPrincipal =
+        openWindowInfo->mPrincipalToInheritForAboutBlank =
             NullPrincipal::Create(parentBC->OriginAttributesRef());
       } else {
-        newWindowPrincipal = NullPrincipal::CreateWithoutOriginAttributes();
+        openWindowInfo->mPrincipalToInheritForAboutBlank =
+            NullPrincipal::CreateWithoutOriginAttributes();
       }
     } else if (aForceNoOpener) {
       // If we're opening a new window with noopener, create a new opaque
       // principal for the new window, rather than re-using the existing
       // principal.
-      newWindowPrincipal =
+      openWindowInfo->mPrincipalToInheritForAboutBlank =
           NullPrincipal::CreateWithInheritedAttributes(subjectPrincipal);
     } else {
       // Finally, if there's an opener relationship and it's not a special
       // principal, we should inherit that principal for the new window.
-      newWindowPrincipal = subjectPrincipal;
+      openWindowInfo->mPrincipalToInheritForAboutBlank = subjectPrincipal;
     }
   }
 
-  // Information used when opening new content windows. This object will be
-  // passed through to the inner nsFrameLoader.
-  RefPtr<nsOpenWindowInfo> openWindowInfo;
   if (!targetBC && !windowTypeIsChrome) {
-    openWindowInfo = new nsOpenWindowInfo();
     openWindowInfo->mForceNoOpener = aForceNoOpener;
     openWindowInfo->mParent = parentBC;
     openWindowInfo->mIsForPrinting = aPrintKind != PRINT_NONE;
@@ -936,21 +932,17 @@ nsresult nsWindowWatcher::OpenWindowInternal(
     openWindowInfo->mIsRemote = XRE_IsContentProcess();
 
     // Inherit our OriginAttributes from the computed new window principal.
-    MOZ_ASSERT(
-        newWindowPrincipal &&
-        !nsContentUtils::IsSystemOrExpandedPrincipal(newWindowPrincipal));
-    openWindowInfo->mOriginAttributes =
-        newWindowPrincipal->OriginAttributesRef();
+    MOZ_ASSERT(openWindowInfo->mPrincipalToInheritForAboutBlank &&
+               !nsContentUtils::IsSystemOrExpandedPrincipal(
+                   openWindowInfo->mPrincipalToInheritForAboutBlank));
 
     MOZ_DIAGNOSTIC_ASSERT(
-        !parentBC || openWindowInfo->mOriginAttributes.EqualsIgnoringFPD(
+        !parentBC || openWindowInfo->GetOriginAttributes().EqualsIgnoringFPD(
                          parentBC->OriginAttributesRef()),
         "subject principal origin attributes doesn't match opener");
   }
 
   uint32_t activeDocsSandboxFlags = 0;
-  nsCOMPtr<nsIPolicyContainer> policyContainerToInheritForAboutBlank;
-  Maybe<nsILoadInfo::CrossOriginEmbedderPolicy> coepToInheritForAboutBlank;
   if (!targetBC) {
     // We're going to either open up a new window ourselves or ask a
     // nsIWindowProvider for one.  In either case, we'll want to set the right
@@ -964,8 +956,17 @@ nsresult nsWindowWatcher::OpenWindowInternal(
       activeDocsSandboxFlags = parentDoc->GetSandboxFlags();
 
       if (!aForceNoOpener) {
-        policyContainerToInheritForAboutBlank = parentDoc->GetPolicyContainer();
-        coepToInheritForAboutBlank = parentDoc->GetEmbedderPolicy();
+        // Inherit from the entry global, e.g. for window.open and fall
+        // back to the parent, e.g. for link clicks.
+        Document* creator = GetEntryDocument();
+        if (!creator) {
+          creator = parentDoc;
+        }
+        openWindowInfo->mPolicyContainerToInheritForAboutBlank =
+            creator->GetPolicyContainer();
+        openWindowInfo->mCoepToInheritForAboutBlank =
+            creator->GetEmbedderPolicy();
+        openWindowInfo->mBaseUriToInheritForAboutBlank = creator->GetBaseURI();
       }
 
       // Check to see if this frame is allowed to navigate, but don't check if
@@ -1004,7 +1005,6 @@ nsresult nsWindowWatcher::OpenWindowInternal(
               windowIsNew = false;
             }
           }
-
         } else if (rv == NS_ERROR_ABORT) {
           // NS_ERROR_ABORT means the window provider has flat-out rejected
           // the open-window call and we should bail.  Don't return an error
@@ -1091,8 +1091,10 @@ nsresult nsWindowWatcher::OpenWindowInternal(
          completely honest: we clear that indicator if the opener is chrome, so
          that the downstream consumer can treat the indicator to mean simply
          that the new window is subject to popup control. */
-      rv = CreateChromeWindow(parentChrome, chromeFlags, openWindowInfo,
-                              getter_AddRefs(newChrome));
+      rv = CreateChromeWindow(
+          parentChrome, chromeFlags,
+          windowTypeIsChrome ? nullptr : openWindowInfo.get(),
+          getter_AddRefs(newChrome));
       if (parentTopInnerWindow) {
         parentTopInnerWindow->Resume();
       }
@@ -1233,8 +1235,8 @@ nsresult nsWindowWatcher::OpenWindowInternal(
   if (windowIsNew) {
     MOZ_DIAGNOSTIC_ASSERT(
         !targetBC->IsContent() ||
-        newWindowPrincipal->OriginAttributesRef().EqualsIgnoringFPD(
-            targetBC->OriginAttributesRef()));
+        openWindowInfo->mPrincipalToInheritForAboutBlank->OriginAttributesRef()
+            .EqualsIgnoringFPD(targetBC->OriginAttributesRef()));
 
     bool autoPrivateBrowsing = StaticPrefs::browser_privatebrowsing_autostart();
 
@@ -1264,16 +1266,28 @@ nsresult nsWindowWatcher::OpenWindowInternal(
     NS_ASSERTION(targetOuterWin == targetDocShell->GetWindow(),
                  "Different windows??");
 
-    // Initialize the principal of the initial about:blank document. For
-    // toplevel windows, this call may have already happened when the window was
-    // created, but SetInitialPrincipal is safe to call multiple times.
     if (targetOuterWin) {
       MOZ_ASSERT(windowIsNew);
       MOZ_ASSERT(!targetOuterWin->GetSameProcessOpener() ||
                  targetOuterWin->GetSameProcessOpener() == aParent);
-      targetOuterWin->SetInitialPrincipal(newWindowPrincipal,
-                                          policyContainerToInheritForAboutBlank,
-                                          coepToInheritForAboutBlank);
+      Document* doc = targetBC->GetExtantDocument();
+      if (doc) {
+        // Previously, the principal was set here. Assert that we already have
+        // the principal that we would have previously set here.
+        MOZ_ASSERT(doc->GetPrincipal()->Equals(
+                       openWindowInfo->mPrincipalToInheritForAboutBlank) ||
+                       (doc->GetPrincipal()->GetIsNullPrincipal() &&
+                        openWindowInfo->mPrincipalToInheritForAboutBlank
+                            ->GetIsNullPrincipal()),
+                   "Wrong principal!");
+        // Setting the principal would've caused a location change event and
+        // frontend code depends on that for setting browser.documentURI.
+        if (nsIURI* uri = doc->GetDocumentURI()) {
+          targetDocShell->FireOnLocationChange(targetDocShell, nullptr, uri, 0);
+        }
+      } else {
+        MOZ_ASSERT_UNREACHABLE("How come there is no doc?");
+      }
 
       if (aIsPopupSpam) {
         MOZ_ASSERT(!targetBC->GetIsPopupSpam(),
@@ -1316,19 +1330,31 @@ nsresult nsWindowWatcher::OpenWindowInternal(
       targetBC->UseRemoteSubframes() ==
       !!(chromeFlags & nsIWebBrowserChrome::CHROME_FISSION_WINDOW));
 
-  if (aLoadState) {
+  // We need to distinguish the case where the no-URL-argument case behaves like
+  // an explicit about:blank argument.
+  RefPtr<nsDocShellLoadState> loadState = aLoadState;
+  nsCOMPtr<nsIURI> uriToLoad = aUri;
+  if (windowIsNew && !uriToLoad && aCalledFromJS && !loadState) {
+    NS_NewURI(getter_AddRefs(uriToLoad), "about:blank"_ns);
+    // Create loadState lazily for about:blank instead of before consuming
+    // user activation in nsGlobalWindowOuter::OpenInternal. See Bug 1901139
+    loadState = CreateLoadState(
+        uriToLoad, aParent ? nsPIDOMWindowOuter::From(aParent) : nullptr);
+  }
+
+  if (loadState) {
     // TriggeringPrincipal and ReferrerInfo are set up here because we
     // rely on the `jsapiChromeGuard` set above to get proper value.
-    // Ideally, aLoadState should contain that value when passed in.
-    if (!aLoadState->TriggeringPrincipal()) {
-      aLoadState->SetTriggeringPrincipal(subjectPrincipal);
+    // Ideally, loadState should contain that value when passed in.
+    if (!loadState->TriggeringPrincipal()) {
+      loadState->SetTriggeringPrincipal(subjectPrincipal);
 #ifndef ANDROID
       MOZ_ASSERT(subjectPrincipal,
                  "nsWindowWatcher: triggeringPrincipal required");
 #endif
     }
 
-    if (!aLoadState->GetReferrerInfo() && !aForceNoReferrer) {
+    if (!loadState->GetReferrerInfo() && !aForceNoReferrer) {
       /* use the URL from the *extant* document, if any. The usual accessor
          GetDocument will synchronously create an about:blank document if
          it has no better answer, and we only care about a real document.
@@ -1341,7 +1367,7 @@ nsresult nsWindowWatcher::OpenWindowInternal(
       }
       if (doc) {
         auto referrerInfo = MakeRefPtr<ReferrerInfo>(*doc);
-        aLoadState->SetReferrerInfo(referrerInfo);
+        loadState->SetReferrerInfo(referrerInfo);
       }
     }
 
@@ -1350,7 +1376,7 @@ nsresult nsWindowWatcher::OpenWindowInternal(
       if (win) {
         nsCOMPtr<nsIPolicyContainer> policyContainer =
             win->GetPolicyContainer();
-        aLoadState->SetPolicyContainer(policyContainer);
+        loadState->SetPolicyContainer(policyContainer);
       }
     }
   }
@@ -1380,10 +1406,10 @@ nsresult nsWindowWatcher::OpenWindowInternal(
     if (obsSvc) {
       RefPtr<nsHashPropertyBag> props = new nsHashPropertyBag();
 
-      if (aUri) {
+      if (uriToLoad) {
         // The url notified in the webNavigation.onCreatedNavigationTarget
         // event.
-        props->SetPropertyAsACString(u"url"_ns, aUri->GetSpecOrDefault());
+        props->SetPropertyAsACString(u"url"_ns, uriToLoad->GetSpecOrDefault());
       }
 
       props->SetPropertyAsInterface(u"sourceTabDocShell"_ns, parentDocShell);
@@ -1396,7 +1422,7 @@ nsresult nsWindowWatcher::OpenWindowInternal(
     }
   }
 
-  if (aLoadState) {
+  if (loadState) {
     uint32_t loadFlags = nsIWebNavigation::LOAD_FLAGS_NONE;
     if (windowIsNew) {
       loadFlags |= nsIWebNavigation::LOAD_FLAGS_FIRST_LOAD;
@@ -1413,11 +1439,11 @@ nsresult nsWindowWatcher::OpenWindowInternal(
         loadFlags |= nsIWebNavigation::LOAD_FLAGS_DISALLOW_INHERIT_PRINCIPAL;
       }
     }
-    aLoadState->SetLoadFlags(loadFlags);
-    aLoadState->SetFirstParty(true);
+    loadState->SetLoadFlags(loadFlags);
+    loadState->SetFirstParty(true);
 
     // Should this pay attention to errors returned by LoadURI?
-    targetBC->LoadURI(aLoadState);
+    targetBC->LoadURI(loadState);
   }
 
   if (windowIsModal) {
@@ -1623,14 +1649,7 @@ nsWindowWatcher::AddWindow(mozIDOMWindowProxy* aWindow,
     // its chrome mapping and return
     info = FindWindowEntry(aWindow);
     if (info) {
-      nsCOMPtr<nsISupportsWeakReference> supportsweak(
-          do_QueryInterface(aChrome));
-      if (supportsweak) {
-        supportsweak->GetWeakReference(getter_AddRefs(info->mChromeWeak));
-      } else {
-        info->mChrome = aChrome;
-        info->mChromeWeak = nullptr;
-      }
+      info->mChrome = do_GetWeakReference(aChrome);
       return NS_OK;
     }
 
@@ -1684,7 +1703,8 @@ nsWatcherWindowEntry* nsWindowWatcher::FindWindowEntry(
   info = mOldestWindow;
   listEnd = nullptr;
   while (info != listEnd) {
-    if (info->mWindow == aWindow) {
+    nsCOMPtr<mozIDOMWindowProxy> window = do_QueryReferent(info->mWindow);
+    if (window && window == aWindow) {
       return info;
     }
     info = info->mYounger;
@@ -1715,8 +1735,11 @@ nsresult nsWindowWatcher::RemoveWindow(nsWatcherWindowEntry* aInfo) {
   // send notifications.
   nsCOMPtr<nsIObserverService> os = mozilla::services::GetObserverService();
   if (os) {
-    nsCOMPtr<nsISupports> domwin(do_QueryInterface(aInfo->mWindow));
-    os->NotifyObservers(domwin, "domwindowclosed", nullptr);
+    nsCOMPtr<mozIDOMWindowProxy> window = do_QueryReferent(aInfo->mWindow);
+    if (window) {
+      nsCOMPtr<nsISupports> domwin(do_QueryInterface(window));
+      os->NotifyObservers(domwin, "domwindowclosed", nullptr);
+    }
   }
 
   delete aInfo;
@@ -1734,12 +1757,8 @@ nsWindowWatcher::GetChromeForWindow(mozIDOMWindowProxy* aWindow,
   MutexAutoLock lock(mListLock);
   nsWatcherWindowEntry* info = FindWindowEntry(aWindow);
   if (info) {
-    if (info->mChromeWeak) {
-      return info->mChromeWeak->QueryReferent(
-          NS_GET_IID(nsIWebBrowserChrome), reinterpret_cast<void**>(aResult));
-    }
-    *aResult = info->mChrome;
-    NS_IF_ADDREF(*aResult);
+    nsCOMPtr<nsIWebBrowserChrome> chrome = do_QueryReferent(info->mChrome);
+    chrome.forget(aResult);
   }
   return NS_OK;
 }
@@ -1856,6 +1875,9 @@ bool nsWindowWatcher::ShouldOpenPopup(const WindowFeatures& aFeatures) {
  * from a child process. The feature string can only control whether to open a
  * new tab or a new popup.
  * @param aFeatures a string containing a list of named features
+ * @param aCalledFromJS a bool indicating whether the features were provided by
+ content JS. If not, we can expose non-standard, more powerful features to
+ content callers.
  * @param aIsPopupRequested an out parameter that indicates whether a popup
  *        is requested by aFeatures
  * @return the chrome bitmask
@@ -1864,7 +1886,12 @@ bool nsWindowWatcher::ShouldOpenPopup(const WindowFeatures& aFeatures) {
 uint32_t nsWindowWatcher::CalculateChromeFlagsForContent(
     const WindowFeatures& aFeatures,
     const mozilla::dom::UserActivation::Modifiers& aModifiers,
-    bool* aIsPopupRequested) {
+    bool aCalledFromJS, bool* aIsPopupRequested) {
+  if (!aCalledFromJS &&
+      aFeatures.GetBoolWithDefault("pictureinpicture", false)) {
+    return nsIWebBrowserChrome::CHROME_DOCUMENT_PICTURE_IN_PICTURE;
+  }
+
   if (aFeatures.IsEmpty() || !ShouldOpenPopup(aFeatures)) {
     // Open the current/new tab in the current/new window
     // (depends on browser.link.open_newwindow).
@@ -2064,7 +2091,7 @@ bool nsWindowWatcher::HaveSpecifiedSize(const WindowFeatures& features) {
 
 /* static */
 already_AddRefed<nsDocShellLoadState> nsWindowWatcher::CreateLoadState(
-    nsIURI* aUri, nsPIDOMWindowOuter* aParent) {
+    nsIURI* aUri, nsPIDOMWindowOuter* aParent, bool aIsWindowOpen) {
   MOZ_ASSERT(aUri);
 
   RefPtr<nsDocShellLoadState> loadState = new nsDocShellLoadState(aUri);
@@ -2094,6 +2121,12 @@ already_AddRefed<nsDocShellLoadState> nsWindowWatcher::CreateLoadState(
     }
   }
 
+  // If we're called from JS, i.e window.open, we need to set history handling
+  // behavior here to be able to do push to replace conversion if needed.
+  if (aIsWindowOpen && mozilla::SessionHistoryInParent()) {
+    loadState->SetHistoryBehavior(NavigationHistoryBehavior::Auto);
+  }
+
   return loadState.forget();
 }
 
@@ -2627,6 +2660,12 @@ int32_t nsWindowWatcher::GetWindowOpenLocation(
       }
     }
   }
+
+  if ((aChromeFlags &
+       nsIWebBrowserChrome::CHROME_DOCUMENT_PICTURE_IN_PICTURE) ==
+      nsIWebBrowserChrome::CHROME_DOCUMENT_PICTURE_IN_PICTURE) {
+    return nsIBrowserDOMWindow::OPEN_NEWWINDOW;
+  }
 #endif
 
   return containerPref;
